#include <stdint.h>
#include <stdio.h>
#include <string.h> 
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/iocan128.h>
#include <avr/iocanxx.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include "define.h"
#include "global.h"
#include "twi-master.h"
#include "delay.h"


/*
 *this function initializes the UART 0 register needed to allow the at90can128 
 *USART transmissions
 */


 int8_t UART0_Init (void){
	 uint8_t intstate1 = SREG;/*save global interrupt flag*/
	 cli();/*disable interrupt*/
	 while(0<<TXC0){
	 }
	 /* set baud rate*/
	 UBRR0H = (uint8_t)(BAUD>>8);
	 UBRR0L = (uint8_t)(BAUD);
	 /* set frame format: i wil set it now as 8 bits, no parity 1 stop bit, asynchron*/
	 UCSR0C = (0<<UMSEL0)|(0<<UPM0)|(0<<USBS0)|(3<<UCSZ0);
         /*set double speed*/
         UCSR0A =(1<<U2X0);
	 /*enable receiver and transmitter; enable RX and Data register empty interrupt*/
	 UCSR0B = (1<<TXEN0)|(1<<RXEN0)|(1<<RXCIE0);
	 SREG=intstate1; /*restore global interrupt flag*/
	 return 1;
 }
  

 

 /* interrupt for receive  data via serial communication with defined interrupt vector SIG_UART0_RECV */  
 ISR (SIG_UART0_RECV)
 {  
	 unsigned char c = UDR0;
	 
	 if( c == '\n' ) /* the string is complete? */
	 {   
		
		 uartString[nextCharPos] = '\0';    
		 uartReady = 1;  /* mark, that we got an CAN_interrupt, to be handled by main */
		 nextCharPos = 0;	
	 }

	 else 
	 {
		 uartString[nextCharPos] = c;
		 nextCharPos++;
		 uartString[nextCharPos] = '\0';
	 }
 }
 
 
 /* interrupt for overflow with defined interrupt vector TIMER0_OVF_vect */

ISR(TIMER0_OVF_vect){

		static  uint8_t  counter0 = 0;  /* counter for timer0 */
		counter0++;

		if(counter0 == 10)  /* 10 overflow for timer0=0.5s */{

			timer0Ready = 1; /* mark to be handled by main */
			counter0 = 0;
		}
	
}//END of ISR(TIMER0_OVF_vect)

void Reset (void)
{
}


void WDF_off (void)
{
  /* Write logical one to WDCE and WDE */
  WDTCR = (1<<WDCE) | (1<<WDE);
  /* Turn off WDT */
  WDTCR = 0x00;

}
/*
 *USART0 must be initialized before using this function
 *for use this function, be sure that data is no longer then 8 bit 
 *(single character) this function will send  8 bits from the at90can128 
 */

 void UART0_Transmit(uint8_t data)
 {
	
	 /* wait for empty transmit buffer */
	 while(!(UCSR0A & (1<<UDRE0)))
	 {
		 /* do nothing*/
	 }
	 /* put the data into the buffer */

	 UDR0 = data;	
	 
 }
 /*
 *this function sends a string to serial communication
 * the input parameter is a defined global vaviable tempstr
 * the output parameter is an integer
 * 0 -> the message is sent
 */
 
 int8_t UART0_Send_Message_String(char *tempstr){
	 
	 for ( uint16_t j = 0; j < strlen ((char *) tempstr) && j <  BUFFER_SIZE; j++)
	 {
		 UART0_Transmit (tempstr[j]);
	 }
	 UART0_Transmit ('\n');
	 memset(tempstr,0,sizeof(tempstr)); /*clear tempstr variable*/
	 return 0;
 }
 

 